# SPDX-FileCopyrightText: 2024-present Helder Guerreiro <helder@tretas.org>
#
# SPDX-License-Identifier: GPL-3.0-or-later

from .__about__ import __version__


def get_version():
    return __version__


def get_name():
    return __name__
