<ul>
{%- for event in event_list -%}
  <li>{{event.date}} - {{event.summary}}{% if show_uid %} - {{event.uid}}{% endif %}{% if show_description %}{{event.description}}{% endif %}</li>{{newline}}
{%- endfor -%}
</ul>
