<ul>{% for todo in todo_list %}
<li>{{todo.status}} -
  <span class="percent">{{todo.percent}}%</span>
  <span class="summary">{{todo.summary}}</span>
  <span class="description">{{todo.description}}</span>
  <span class="uid">{{todo.uid}}</span>
</li>
{% endfor %}</ul>
